# Props

The props listed on this page are the props that `reactive-form` generates to give to your form component.

If you are a strict PropTypes completionist, `reactive-form` exports all of these propTypes, so you may import them.

For every created control you'll receive mapped control props.

Example
```ts
import { 
  FormProps, 
  FormGroup, 
  FormControl,
  reactiveForm
} from "react-reactive-form";
//...
const registerForm = new FormGroup({
  username: new FormControl(''),
  password: new FormControl(''),
  // Nested Form
  address: new FormGroup({
    city: new FormControl("Bangalore"),
    country: new FormControl("India"),
  })
});
//...
class MyForm extends Component {
  static propTypes = {
    username: FormProps,
    password: FormProps,
    address: {
      city: FormProps,
      country: FormProps,
      ...FormProps
    }
    // Some other props
  }
  // ...
}
export default reactiveForm(MyForm, registerForm);
```
## Members
```ts
value: any;
```
The value of the control.
##
```ts
touched: boolean;
```
A control is marked `touched` once the user has triggered a `blur` event on it.
##
```ts
untouched: boolean;
```
A control is `untouched` if the user has not yet triggered a `blur` event on it.
##
```ts
disabled: boolean;
```
A control is disabled when its `status === DISABLED`.
##
```ts
enabled: boolean;
```
A control is enabled as long as its `status !== DISABLED`.
##
```ts
invalid: boolean;
```
A control is invalid when its `status === INVALID`.
##
```ts
valid: boolean;
```
A control is valid when its `status === VALID`.
##
```ts
pristine: boolean;
```
A control is `pristine` if the user has not yet changed the value in the UI.
##
```ts
dirty: boolean;
```
A control is `dirty` if the user has changed the value in the UI.
##
```ts
errors: ValidationErrors;
```
Returns any errors generated by failing validation. If there are no errors, it will return null.
##
```ts
status: Status;
```
The validation status of the control. There are four possible validation statuses:

* VALID: control has passed all validation checks
* INVALID: control has failed at least one validation check
* PENDING: control is in the midst of conducting a validation check
* DISABLED: control is exempt from validation checks
These statuses are mutually exclusive, so a control cannot be both valid AND invalid or invalid AND disabled.
##
```ts
pending: boolean;
```
A control is pending when its `status === PENDING`.
##
```ts
hasError: (errorCode: string, path?: String|Number[]|String) => boolean;
```
Returns true if the control with the given path has the error specified. Otherwise returns false.

If no path is given, it checks for the error on the present control.
##
```ts
getError: (errorCode: string, path: String|Number[]|String) => any;
```
Returns error data if the control with the given path has the error specified. Otherwise returns null or undefined.

If no path is given, it checks for the error on the present control.
##
```ts
handler: (inputType?: InputType, value?: string) => Handler;
```
Returns the props required to bind a control to a HTML input element.

Note: 
* `inputType` parameter is required for `checkbox` and `radio` buttons.
* `value` parameter only can be used for `radio` buttons to assign a value to a particular button.

Example
```ts
  <input type="text" {...username.handler()}/>
```
Binds a `text` input.
##
```ts
  <input type="date" {...birthday.handler()}/> 
```
Binds a `date` input.
##
```ts
<input {...terms.handler("checkbox")}/>
```
Binds a `checkbox` input.
##
```ts
<input type="radio" {...gender.handler('radio', 'male')}/> Male
<input type="radio" {...gender.handler('radio', 'female')}/> Female
<input type="radio" {...gender.handler('radio', 'other')}/> Other
```
Binds a `radio` input.

A `handler` is an object with these properties:

```ts
value: any;
```
Sometimes this value can be different from the actual value of `control`.

For example, if the `updateOn` property is `blur` or `submit` than the value property of handler will be `_pendingValue`
of the control.

The `_pendingValue` is the value of a control which is not validated yet which means the actual value of the
control is different.

So, this `value` is just to control the input element, for actual value of the control you can use the `value` property 
of the mapped control prop.
##
```ts
onChange: (e: any) => void;
```
Function needs to be called whenever a value change event triggers.
##
```ts
onBlur: (e: any) => void;
```
Function needs to be called whenever a `blur` event triggers.
##
```ts
disabled: boolean;
```
Tells the input element about the `disabled` status.
##
```ts
checked?: boolean;
```
Checked property for `checkbox` and `radio` buttons.
##
```ts
editable?: boolean;
```
React Native uses `editable` property to tell the `TextInput` about the `enabled` status.
##
```ts
type?: string;
```
Returns the type of input element in case of `checkbox` & `radio` buttons.


Although `handler` works well with all kind of inputs, you can also use it to bind your custom input 
components.
